import 'package:local_auth/local_auth.dart';
import 'package:biometric_storage/biometric_storage.dart';

class SecureAuthService {
  final LocalAuthentication auth = LocalAuthentication();

  Future<bool> authenticate() async {
    bool isBiometricSupported = await auth.canCheckBiometrics &&
                                 await auth.isDeviceSupported();
    if (!isBiometricSupported) {
      // Handle the case where biometrics are not supported
      return false;
    }

    try {
      return await auth.authenticate(
        localizedReason: 'Authenticate to access secure storage',
        options: const AuthenticationOptions(biometricOnly: true),
      );
    } catch (e) {
      // Handle authentication errors or user cancellation
      return false;
    }
  }

  Future<BiometricStorageFile?> getSecureStorage(String key) async {
    if (await authenticate()) {
      try {
        return BiometricStorage().getStorage(
          key,
          options: StorageFileInitOptions(
            authenticationValidityDurationSeconds: 10, // Optional: Re-authenticate after a timeout
          ),
        );
      } catch (e) {
        // Handle errors in accessing or initializing the secure storage
        return null;
      }
    } else {
      // Authentication failed or was cancelled
      return null;
    }
  }

  Future<void> storeKey(String key, String value) async {
    final storage = await getSecureStorage(key);
    if (storage != null) {
      await storage.write(value);
      // Optionally: Provide feedback to the user about the successful operation
    } else {
      // Handle the case where the storage could not be accessed
    }
  }

  Future<String?> retrieveKey(String key) async {
    final storage = await getSecureStorage(key);
    if (storage != null) {
      return await storage.read();
    } else {
      // Handle the case where the storage could not be accessed
      return null;
    }
  }
}
