import 'package:flutter_secure_storage/flutter_secure_storage.dart';
import 'package:local_auth/local_auth.dart';

class SecureAuthService {
  final LocalAuthentication auth = LocalAuthentication();
  final FlutterSecureStorage _secureStorage = const FlutterSecureStorage();

  Future<bool> authenticate() async {
    bool isBiometricSupported = await auth.canCheckBiometrics &&
                               await auth.isDeviceSupported();
    if (!isBiometricSupported) {
      // Handle the case where biometrics are not supported
      return false;
    }

    try {
      return await auth.authenticate(
        localizedReason: 'Authenticate to access secure storage',
        options: const AuthenticationOptions(biometricOnly: true),
      );
    } catch (e) {
      // Handle authentication errors or user cancellation
      return false;
    }
  }

  Future<void> storeKey(String key, String value) async {
    if (await authenticate()) {
      try {
        await _secureStorage.write(key: key, value: value);
        // Optionally: Provide feedback to the user about the successful operation
      } catch (e) {
        // Handle errors in accessing or initializing the secure storage
      }
    } else {
      // Authentication failed or was cancelled
    }
  }

  Future<String?> retrieveKey(String key) async {
    if (await authenticate()) {
      try {
        return await _secureStorage.read(key: key);
      } catch (e) {
        // Handle errors in accessing or initializing the secure storage
        return null;
      }
    } else {
      // Authentication failed or was cancelled
      return null;
    }
  }


  Future<Map<String, String>?> retrieveKeys(Map<String, dynamic> keys) async {
    if (await authenticate()) {
      try {
        Map<String, String> returnItems = {};
        for (var entry in keys.entries) {
          String? value = await _secureStorage.read(key: entry.key);
          if (value != null) {
            returnItems[entry.key] = value;
          }
        }
        return returnItems;
      } catch (e) {
        return null;
      }
    } else {
      return null;
    }
  }

}